import webbrowser
from sys import stdin, exit

import click

from desec_dns_cli.util import get_config, save_config, get_account_info
from desec_dns_api import UnexpectedError, FieldError, UnauthorizedError, NonEmptyAccountError
from desec_dns_api.account import obtain_captcha, register, reset_password, login, info, delete_account, logout


@click.group(name='account')
def _account():
    pass


@click.command(name='register')
@click.option('-d', '--domain', type=str, required=False, help='DNS zone that should be created with the account.')
@click.option('-p', '--password-from-stdin', 'passwordfromstdin', is_flag=True, flag_value=True, required=False,
              help='Read password from standard input.')
@click.argument('email', type=str, required=True)
@click.argument('password', type=str, required=False)
def _register(domain: str, passwordfromstdin: bool, email: str, password: str):
    """
    Register an account with the deSEC DNS API.
    """
    if passwordfromstdin:
        click.echo('Reading password from stdin..')
        password = stdin.read()

    if not password:
        password = click.prompt(text='Enter password for the account', hide_input=True, type=str)

    click.echo('Optaining captcha..')
    data = obtain_captcha()

    # TODO: handle headless environments https://gitlab.com/XenGi/desec-dns-cli/issues/1
    click.echo('Opening captcha in browser..')
    webbrowser.open_new_tab(url=f'data:image/png;base64,{ data["challenge"] }')

    solution = click.prompt(text='Enter captcha text', type=str)
    try:
        register(email=email, password=password, captcha_id=data['id'], captcha_solution=solution, domain=domain)
        click.echo('If the account doesn\'t already exist, you will receive an email with a verification link. To'
                   'activate your account, click on that link. The link expires after 12 hours.')
    except FieldError as e:
        click.echo(f'Failed to create account:\n{ e }', err=True)
        exit(1)
    except UnexpectedError as e:
        click.echo(f'Unexpected error: { e }', err=True)
        exit(1)


@click.command(name='login')
@click.option('-p', '--password-from-stdin', 'password_from_stdin', is_flag=True, flag_value=True, required=False,
              help='Read password from standard input.')
@click.option('-s', '--save', is_flag=True, flag_value=True, required=False, help='Save auth_token to config file.')
@click.argument('email', type=str, required=True)
@click.argument('password', type=str, required=False)
def _login(password_from_stdin: bool, save: bool, email: str, password: str):
    """
    Login and obtain new token.
    """
    if password_from_stdin:
        click.echo('Reading password from stdin..')
        password = stdin.read()

    if not password:
        password = click.prompt(text='Enter password for the account', hide_input=True, type=str)

    try:
        token = login(email=email, password=password)

        if save:
            conf = get_config()
            if not conf.has_section('auth'):
                conf.add_section('auth')
            conf.set('auth', 'token', token)
            save_config(conf)
        else:
            click.echo(f'token={ token }')
    except (IOError, UnexpectedError) as e:
        click.echo(f'Unexpected error: { e }', err=True)
        exit(1)
    except UnauthorizedError as e:
        click.echo(f'Failed to login: { e }', err=True)
        exit(1)


@click.command(name='info')
@click.option('-f', '--format', type=str, required=False, default='text', show_default='text',
              help='Set output format to "json", "yaml" or "text".')
def _info(format: str):
    """
    Show account info.
    """
    account_info = get_account_info()
    if format == 'json':
        click.echo('{\n'
                  f'  "id": "{ account_info["id"] }",\n'
                  f'  "email": "{ account_info["email"] }",\n'
                  f'  "created": "{ account_info["created"] }",\n'
                  f'  "limit_domains": { account_info["limit_domains"] }\n'
                   '}')
    elif format == 'yaml':
        click.echo('---\n'
                  f'id: "{ account_info["id"] }"\n'
                  f'email: "{ account_info["email"] }"\n'
                  f'created: "{ account_info["created"] }"\n'
                  f'limit_domains: { account_info["limit_domains"] }')
    else:
        click.echo(f'ID: { account_info["id"] }\n'
                   f'Email: { account_info["email"] }\n'
                   f'Created: { account_info["created"] }\n'
                   f'Limit domains: { account_info["limit_domains"] }')


@click.command(name='reset-password')
@click.argument('email', type=str, required=False)
def _reset_password(email: str):
    """
    Request password reset.
    """
    if not email:
        account_info = get_account_info()
        email = account_info["email"]

    click.echo('Optaining captcha..')
    data = obtain_captcha()

    # TODO: handle headless environments https://gitlab.com/XenGi/desec-dns-cli/issues/1
    click.echo('Opening captcha in browser..')
    webbrowser.open_new_tab(url=f'data:image/png;base64,{ data["challenge"] }')

    solution = click.prompt(text='Enter captcha text', type=str)
    try:
        reset_password(email=email, captcha_id=data['id'], captcha_solution=solution)
        click.echo('If the account exists, you will receive an email with a verification link. To change your password,'
                   'click on that link. The link expires after 12 hours.')
    except FieldError as e:
        click.echo(f'Failed send request:\n{ e }', err=True)
        exit(1)
    except UnexpectedError as e:
        click.echo(f'Unexpected error: { e }', err=True)
        exit(1)


# TODO: change email https://gitlab.com/XenGi/desec-dns-cli/issues/7


@click.command(name='delete')
@click.option('-p', '--password-from-stdin', 'password_from_stdin', is_flag=True, flag_value=True, required=False,
              help='Read password from standard input.')
@click.argument('email', type=str, required=True)
@click.argument('password', type=str, required=False)
def _delete(password_from_stdin: bool, email: str, password: str):
    """
    Delete your deSEC account.
    """
    if password_from_stdin:
        click.echo('Reading password from stdin..')
        password = stdin.read()

    if not password:
        password = click.prompt(text='Enter password for the account', hide_input=True, type=str)

    try:
        delete_account(email=email, password=password)
    except NonEmptyAccountError:
        click.echo('Can\'t delete account which still has domains in it.')
    except UnauthorizedError as e:
        click.echo(f'Failed to delete account: { e }')
    except UnexpectedError as e:
        click.echo(f'Unexpected error: { e }')


@click.command(name='logout')
@click.argument('token', type=str, required=True)
def _logout(token: str):
    """
    Logout and invalidate an authentication token.
    """
    try:
        logout(token=token)
    except UnexpectedError as e:
        click.echo(f'Unexpected error: { e }')


_account.add_command(_register)
_account.add_command(_login)
_account.add_command(_info)
_account.add_command(_reset_password)
_account.add_command(_delete)
_account.add_command(_logout)
