import os
from typing import Dict
from configparser import ConfigParser

from click import echo

from desec_dns_api import UnexpectedError, UnauthorizedError
from desec_dns_api.account import info


CONFIG_FILE_PATH = '~/.config/desec-dns-cli.conf'


def get_config(configfile: str = CONFIG_FILE_PATH) -> ConfigParser:
    """
    Read config file and return config dict.

    :param configfile: Path to config file
    :return: Parsed config
    """
    config = ConfigParser()
    if os.path.exists(os.path.expanduser(configfile)):
        config.read(os.path.expanduser(configfile))

    return config


def save_config(config: ConfigParser, configfile: str = CONFIG_FILE_PATH):
    """
    Save configuration back to default location.

    :param config: The configuration to save
    :param configfile: Path to config file
    :return:
    """
    with open(os.path.expanduser(configfile), 'w') as fh:
        config.write(fh)



def get_account_info() -> Dict:
    """
    Get account infos from API using token from config.

    :return: Account info from API
    """
    conf = get_config()
    if not conf.has_option('auth', 'token'):
        echo('Can\'t find "token" in config file.', err=True)
        exit(1)

    token = conf.get('auth', 'token')

    try:
        return info(token=token)
    except UnauthorizedError:
        echo('Token from config is invalid', err=True)
        exit(1)
    except UnexpectedError as e:
        echo(f'Unexpected error: { e }', err=True)
        exit(1)
